'use strict';

/**
 * @module gulp tasks
 */

var gulp = require('gulp');
var childProcess = require('child_process');
var Mocha = require('mocha');
var vinylPaths = require('vinyl-paths');

// Show more error detail.
gulp.onAll(function(e) {
  if (!e.err) return;
  console.error('Gulp caught error: '+e.err.stack);
});

process.on('uncaughtException', function(error) {
  console.error('Gulp caught error: '+error.stack);
});

/**
 * @name docs
 * Cleans and regenerates docs using jsdoc
 */
gulp.task('docs', function(done) {
  childProcess.spawnSync('rm', ['-rf', 'docs'], {
    stdio: 'inherit',
    cwd: __dirname
  });

  childProcess.spawnSync('jsdoc', [
    '-r',
    '-d', './docs/',
    '-a', 'all',
    '-R', './README.md', 'gulpfile.js', 'lib', 'test'
  ], {
    stdio: 'inherit',
    cwd: __dirname
  });

  done();
});

/**
 * @name docs-watch
 * Regenerates docs every time js files change.  Good for iterating on docs
 */

gulp.task('docs-watch', function() {
  gulp.watch(['./lib/**/*.js', './test/**/*.js', './gulpfile.js'], ['docs']);
});

/**
 * @name lint
 * Runs eslint on js files
 */

gulp.task('lint', function() {
  var eslint = require('gulp-eslint');
  return gulp.src(['./lib/**/*.js', './test/**/*.js'])
    .pipe(eslint())
    .pipe(eslint.formatEach());
});

var testPaths = {
  acceptance: './test/acceptance/**/*.js',
  functional: './test/functional/**/*.js',
  integration: './test/integration/**/*.js',
  unit: './test/unit/**/*.js'
};

var testTypes = Object.keys(testPaths);

gulp.task('test', function() {
  var paths = testTypes.map(function(testType) {
    return testPaths[testType];
  });

  return runTests(paths);
});

gulp.task('test-all', ['test']);

testTypes.forEach(function(testType) {
  gulp.task('test-'+testType, function() {
    return runTests(testPaths[testType]);
  });
});

function runTests(testPaths) {
  process.env.NODE_ENV = 'test';
  process.env.BLUEBIRD_DEBUG = '1';

  var mocha = new Mocha();

  return gulp.src(['./test/bootstrap/node.js'].concat(testPaths), { read: false })
  .pipe(vinylPaths(function(path) {
    if (path.substr(-3) === '.js') {
      mocha.addFile(path);
    }
    return Promise.resolve();
  }))
  .on('end', function() {
    mocha.run(function(failures) {
      process.exit(failures);
    });
  });
}

gulp.task('default', ['test']);
