'use strict';
var express = require('express');
var logger = require('./logger.js');
var jsonHasher = require('./json-hasher.js');
var queryTemplate = require('./query-template.js');

function RestDataAPI(options) {
  if (!options) throw new Error('Missing options');
  if (!options.getQuery) throw new Error("Missing option 'getQuery'");
  if (!options.runQuery) throw new Error("Missing option 'runQuery'");
  if (!options.upsertQuery) throw new Error("Missing option 'upsertQuery'");

  this.getQuery = options.getQuery;
  this.runQuery = options.runQuery;
  this.upsertQuery = options.upsertQuery;
  this.jsonParser = options.jsonParser || require('body-parser').json();
  this.base = options.base;
}

RestDataAPI.prototype.optionsHandler = function() {
  return function(req, res, next) {
    res.set('Allow', 'GET,OPTIONS,POST');
    res.sendStatus(200);
    next();
  };
};

RestDataAPI.prototype.getHandler = function() {
  return function(req, res, next) {
    return this.getQuery({ hash: req.params.hash })
    .then(function(restQuery) {
      var query = queryTemplate({
        body: JSON.parse(restQuery.queryTemplate),
        params: req.query
      }).query;

      return [
        restQuery,
        this.runQuery({ modelName: restQuery.modelName, query: query })
      ];
    }.bind(this))
    .spread(function(restQuery, results) {
      logger.debug('Results: %j', results);

      res.status(200).send({
        modelName: restQuery.modelName,
        hash: restQuery.hash,
        results: results
      });

      next();
    });
  }.bind(this);
};

RestDataAPI.prototype.postHandler = function() {
  return function(req, res, next) {
    var hash = jsonHasher(req.body).hash;

    return this.upsertQuery({
      queryHash: hash,
      modelName: req.body.modelName,
      queryName: req.body.queryName,
      queryTemplate: JSON.stringify(req.body.queryTemplate)
    })
    .then(function() {
      res.status(200).send({
        hash: hash
      });

      next();
    });
  }.bind(this);
};

RestDataAPI.prototype.middleware = function() {
  if (!this.base) throw new Error("Missing option 'base'");

  var middleware = express.Router();
  middleware.use(this.jsonParser);
  middleware.options(this.base, this.optionsHandler());
  middleware.get(this.base+'/:hash', this.getHandler());
  middleware.post(this.base, this.postHandler());

  return middleware;
};

module.exports = RestDataAPI;
