'use strict';
var uuid = require('uuid');
var mysql = require('promise-mysql');
var logger = require('../../lib/logger.js');
var Schema = require('./todo/schema/');
var Todo = require('./todo/index.js');

var Fixture = function() {
  this.name = uuid.v4().replace(/-/g, '');
  this.dbName = 'rest_query_api_test_'+this.name;
  this.dbUrl = 'mysql://root@localhost/'+this.dbName;

  Fixture.instances[this.name] = this;
};

Fixture.instances = {};

Fixture.prototype.init = function() {
  return Promise.resolve()
  .then(this.createTestDb.bind(this))
  .then(this.createSchema.bind(this));
};

Fixture.prototype.createTestDb = function() {
  return mysql.createConnection({
    host: 'localhost',
    user: 'root',
    debug: !!process.env.MYSQL_DEBUG
  })
  .then(function(connection) {
    this.connection = connection;
    return this.connection.query('CREATE DATABASE `'+this.dbName+'` DEFAULT CHARACTER SET utf8 COLLATE utf8_general_ci;');
  }.bind(this))
  .then(function() {
    logger.info('Created test db '+this.dbName);
    return this.connection.end();
  }.bind(this));
};

Fixture.prototype.createSchema = function() {
  this.schema = new Schema(this.dbUrl, this.logger);
  return this.schema.init();
};

Fixture.prototype.createTodo = function() {
  this.todo = new Todo(this.schema);

  return this.schema.Project.create({
    name: 'Project 1',
    description: '',
    Tasks: [{
      name: 'Task 1',
      description: '',
      completed: true
    }, {
      name: 'Task 2',
      description: '',
      completed: false
    }]
  }, { include: [this.schema.Task] })
  .then(function(project1) {
    this.project1 = project1;
    this.task1 = this.project1.Tasks[0];
    this.task2 = this.project1.Tasks[1];
  }.bind(this))
  .then(function() {
    return this.todo;
  }.bind(this));
};

Fixture.prototype.cleanup = function() {
  return mysql.createConnection({
    host: 'localhost',
    user: 'root',
    debug: !!process.env.MYSQL_DEBUG
  })
  .then(function(connection) {
    this.connection = connection;
    return this.connection.query('DROP DATABASE `'+this.dbName+'`');
  }.bind(this))
  .then(function() {
    logger.info('Dropped test db '+this.dbName);
  }.bind(this))
  .catch(function(err) {
    logger.error('Error encountered while cleaning up fixture %s: %s', this.name, err.stack || err);
    logger.error('You may need to manually remove the MySQL test database named %s', this.dbName);
  }.bind(this))
  .finally(function() {
    delete Fixture.instances[this.name];
  }.bind(this));
};

module.exports = Fixture;
